//
//  Barcode_MobileBarcodeTableViewController.m
//  Barcode Mobile
//
//  Created by Aaron Wright on 5/16/12.
//  Copyright (c) 2012 A.C. Wright Design. All rights reserved.
//

#import "Barcode_MobileBarcodeTableViewController.h"
#import "Barcode_MobileAppDelegate.h"

@implementation Barcode_MobileBarcodeTableViewController

@synthesize barcodes = _barcodes;
@synthesize managedObjectContext = _managedObjectContext;
@synthesize barcode = _barcode;

# pragma mark -
# pragma mark view

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(fetch:) 
                                                 name:kBarcode_MobileUbiquityContentChangedNotification 
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(fetch:) 
                                                 name:NSManagedObjectContextObjectsDidChangeNotification 
                                               object:_managedObjectContext];
    
    [self fetch:self];
    
    if (_barcode) {
        [self select:self];
    } else {
        if ([_barcodes count] > 0) {
            [self setBarcode:[_barcodes objectAtIndex:0]];
            [self select:self];
        }
    }
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:kBarcode_MobileUbiquityContentChangedNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSManagedObjectContextObjectsDidChangeNotification object:_managedObjectContext];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation {
    return YES;
}

# pragma mark actions

- (void)add:(id)sender {
    BarcodeKitBarcode *barcode = [NSEntityDescription insertNewObjectForEntityForName:@"Barcode" inManagedObjectContext:_managedObjectContext];
    [self setBarcode:barcode];
    [self fetch:sender];
    [(Barcode_MobileAppDelegate *)[[UIApplication sharedApplication] delegate] save:self];
}

- (void)fetch:(id)sender {
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Barcode" inManagedObjectContext:_managedObjectContext];
    [fetchRequest setShouldRefreshRefetchedObjects:YES];
    [fetchRequest setSortDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"title" ascending:YES]]];
    [fetchRequest setEntity:entity];
    
    [self setBarcodes:[_managedObjectContext executeFetchRequest:fetchRequest error:nil]];
    
    [self.tableView reloadData];
    
    [self select:sender];
}

- (void)select:(id)sender {
    if (_barcode) {
        for (int i = 0; i < [_barcodes count]; i++) {
            if ([[_barcodes objectAtIndex:i] isEqual:_barcode]) {
                [self.tableView selectRowAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0] animated:NO scrollPosition:UITableViewScrollPositionNone];
                break;
            }
        }
    }
    [self notify:sender];
}

- (void)notify:(id)sender {
    [[NSNotificationCenter defaultCenter] postNotificationName:kBarcode_MobileBarcodeTableViewControllerSelectionChangedNotification object:_barcode];
}

# pragma mark -
# pragma mark UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [[self barcodes] count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"BarcodeCell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    [[cell textLabel] setText:[[[self barcodes] objectAtIndex:indexPath.row] valueForKey:@"title"]];
    [[cell detailTextLabel] setText:[[[self barcodes] objectAtIndex:indexPath.row] valueForKey:@"symbology"]];
    
    return cell;
}

# pragma mark -
# pragma mark UITableViewDelegate

- (void)tableView:(UITableView *)aTableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [_managedObjectContext deleteObject:[_barcodes objectAtIndex:indexPath.row]];
        [(Barcode_MobileAppDelegate *)[[UIApplication sharedApplication] delegate] save:self];
    }
}

- (void)tableView:(UITableView *)tableView didEndEditingRowAtIndexPath:(NSIndexPath *)indexPath {
    [self select:self];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [self setBarcode:[_barcodes objectAtIndex:indexPath.row]];
    [self select:self];
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath {
    [self setBarcode:nil];
    [self select:self];
}

@end
