//
//  Barcode_MobileBarcodeView.m
//  Barcode Mobile
//
//  Created by Aaron Wright on 4/17/12.
//  Copyright (c) 2012 A.C. Wright Design. All rights reserved.
//

#import "Barcode_MobileBarcodeView.h"

@interface Barcode_MobileBarcodeView ()
- (void)drawRulersInContext:(CGContextRef)context inRect:(CGRect)rect;
@end

@implementation Barcode_MobileBarcodeView

@synthesize barcode = _barcode;
@synthesize shouldShowRulers = _shouldShowRulers;

# pragma mark init

- (id)init
{
    self = [super init];
    if (self) {
        [self setShouldShowRulers:NO];
        [self setBackgroundColor:[UIColor clearColor]];
    }
    return self;
}

- (id)initWithBarcode:(BarcodeKitBarcode *)barcode
{
    self = [super initWithFrame:CGRectZero];
    if (self) {
        [self setBarcode:barcode];
    }
    return self;
}

# pragma mark actions

- (void)setBarcode:(BarcodeKitBarcode *)barcode
{
    _barcode = barcode;
    
    float width = ([[barcode width] floatValue] * 0.001) * 72.0;
    float height = ([[barcode height] floatValue] * 0.001) * 72.0;
    CGSize canvasSize = CGSizeMake(width + kBarcode_MobileBarcodeViewPadding, height + kBarcode_MobileBarcodeViewPadding);
    
    [self setFrame:CGRectMake(0, 0, canvasSize.width, canvasSize.height)];
    
    if ([self superview]) {
        CGSize superviewSize = [[self superview] bounds].size;
        
        if (canvasSize.width < superviewSize.width) {
            [self setFrame:CGRectMake(0, 0, superviewSize.width, [self frame].size.height)];
        }
        if (canvasSize.height < superviewSize.height) {
            [self setFrame:CGRectMake(0, 0, [self frame].size.width, superviewSize.height)];
        }
    }
     
    [self setNeedsDisplay];
}

# pragma mark draw

- (void)drawRect:(CGRect)dirtyRect
{
    for (UIView *view in [self subviews]) {
        [view removeFromSuperview];
    }
    
    if ([_barcode valid]) {
        float width = ([[_barcode width] floatValue] * 0.001) * 72.0;
        float height = ([[_barcode height] floatValue] * 0.001) * 72.0;
        CGSize canvasSize = CGSizeMake(width + kBarcode_MobileBarcodeViewPadding, height + kBarcode_MobileBarcodeViewPadding);
        CGRect canvasRect = CGRectMake((self.bounds.size.width * 0.5) - (canvasSize.width / 2),
                                       (self.bounds.size.height * 0.5) - (canvasSize.height / 2),
                                       canvasSize.width,
                                       canvasSize.height);
        
        CGContextRef context = UIGraphicsGetCurrentContext();
        CGContextSaveGState(context);
        
        CGContextSetShadowWithColor(context, CGSizeMake(2.0, 2.0), 5.0, [UIColor blackColor].CGColor);
        CGContextSetFillColorWithColor(context, [UIColor whiteColor].CGColor);
        CGContextAddRect(context, canvasRect);
        CGContextFillPath(context);
        
        CGContextRestoreGState(context);
        CGContextSaveGState(context);
        
        if (_shouldShowRulers) {
            [self drawRulersInContext:context inRect:[self bounds]];
        }
        
        BarcodeKitBarcodeView *barcodeView = [[BarcodeKitBarcodeView alloc] initWithFrame:CGRectZero];
        [barcodeView setBarcode:_barcode];
        
        float x = (self.frame.size.width * 0.5) - (barcodeView.frame.size.width * 0.5);
        float y = (self.frame.size.height * 0.5) - (barcodeView.frame.size.height * 0.5);
        
        [barcodeView setFrame:CGRectMake(x, y, barcodeView.frame.size.width, barcodeView.frame.size.height)];
        
        [self addSubview:barcodeView];
        
        CGContextRestoreGState(context);
    }
}

- (void)drawRulersInContext:(CGContextRef)context inRect:(CGRect)rect
{
    CGContextSaveGState(context);
    CGContextSetAllowsAntialiasing(context, NO);
    
    float width = ([[_barcode width] floatValue] * 0.001) * 72.0;
    float height = ([[_barcode height] floatValue] * 0.001) * 72.0;
    
    CGRect canvasRect = rect;
    CGPoint canvasCenter = CGPointMake(CGRectGetMidX(canvasRect), CGRectGetMidY(canvasRect));
    
    CGRect barcodeRect = CGRectMake(canvasCenter.x - (width * 0.5), canvasCenter.y - (height * 0.5), width, height);
    
    CGPoint rulerHorizontalOrigin = CGPointMake(barcodeRect.origin.x, barcodeRect.origin.y + kBarcode_MobileBarcodeRulerPadding + kBarcode_MobileBarcodeRulerFontSize + barcodeRect.size.height);
    CGRect rulerHorizontalTextRect = CGRectMake(rulerHorizontalOrigin.x, rulerHorizontalOrigin.y - kBarcode_MobileBarcodeRulerIndicatorLength + kBarcode_MobileBarcodeRulerFontSize, width, kBarcode_MobileBarcodeRulerFontSize);
    
    CGPoint rulerVerticalOrigin = CGPointMake(barcodeRect.origin.x - kBarcode_MobileBarcodeRulerPadding, barcodeRect.origin.y);
    CGRect rulerVerticalTextRect = CGRectMake(rulerVerticalOrigin.x - kBarcode_MobileBarcodeRulerFontSize, rulerVerticalOrigin.y, height, kBarcode_MobileBarcodeRulerFontSize);
    
    CGContextSetLineWidth(context, 1.0);
    CGContextSetStrokeColorWithColor(context, [UIColor lightGrayColor].CGColor);
    
    CGPoint horizontalFirstPoint = CGPointMake(rulerHorizontalOrigin.x, rulerHorizontalOrigin.y - kBarcode_MobileBarcodeRulerIndicatorLength);
    CGPoint horizontalSecondPoint = CGPointMake(rulerHorizontalOrigin.x, rulerHorizontalOrigin.y - (kBarcode_MobileBarcodeRulerIndicatorLength * 0.5));
    CGPoint horizontalThirdPoint = CGPointMake(rulerHorizontalOrigin.x + width, rulerHorizontalOrigin.y - (kBarcode_MobileBarcodeRulerIndicatorLength * 0.5));
    CGPoint horizontalFourthPoint = CGPointMake(rulerHorizontalOrigin.x + width, rulerHorizontalOrigin.y - kBarcode_MobileBarcodeRulerIndicatorLength);
    CGPoint horizontalFifthPoint = CGPointMake(rulerHorizontalOrigin.x + width, rulerHorizontalOrigin.y);
    
    CGContextBeginPath(context);
    CGContextMoveToPoint(context, rulerHorizontalOrigin.x, rulerHorizontalOrigin.y);
    CGContextAddLineToPoint(context, horizontalFirstPoint.x, horizontalFirstPoint.y);
    CGContextMoveToPoint(context, horizontalSecondPoint.x, horizontalSecondPoint.y);
    CGContextAddLineToPoint(context, horizontalThirdPoint.x, horizontalThirdPoint.y);
    CGContextMoveToPoint(context, horizontalFourthPoint.x, horizontalFourthPoint.y);
    CGContextAddLineToPoint(context, horizontalFifthPoint.x, horizontalFifthPoint.y);
    CGContextStrokePath(context);
    
    CGPoint verticalFirstPoint = CGPointMake(rulerVerticalOrigin.x - kBarcode_MobileBarcodeRulerIndicatorLength, rulerVerticalOrigin.y);
    CGPoint verticalSecondPoint = CGPointMake(rulerVerticalOrigin.x - (kBarcode_MobileBarcodeRulerIndicatorLength * 0.5), rulerVerticalOrigin.y);
    CGPoint verticalThirdPoint = CGPointMake(rulerVerticalOrigin.x - (kBarcode_MobileBarcodeRulerIndicatorLength * 0.5), rulerVerticalOrigin.y + height);
    CGPoint verticalFourthPoint = CGPointMake(rulerVerticalOrigin.x - kBarcode_MobileBarcodeRulerIndicatorLength, rulerVerticalOrigin.y + height);
    CGPoint verticalFifthPoint = CGPointMake(rulerVerticalOrigin.x, rulerVerticalOrigin.y + height);
    
    CGContextBeginPath(context);
    CGContextMoveToPoint(context, rulerVerticalOrigin.x, rulerVerticalOrigin.y);
    CGContextAddLineToPoint(context, verticalFirstPoint.x, verticalFirstPoint.y);
    CGContextMoveToPoint(context, verticalSecondPoint.x, verticalSecondPoint.y);
    CGContextAddLineToPoint(context, verticalThirdPoint.x, verticalThirdPoint.y);
    CGContextMoveToPoint(context, verticalFourthPoint.x, verticalFourthPoint.y);
    CGContextAddLineToPoint(context, verticalFifthPoint.x, verticalFifthPoint.y);
    CGContextStrokePath(context);
    
    float widthInInches = width / 72.0;
    float heightInInches = height / 72.0;

    UIFont *font = [UIFont fontWithName:kBarcode_MobileBarcodeRulerFontFamily size:kBarcode_MobileBarcodeRulerFontSize];
    
    [[NSString stringWithFormat:@"%f in", widthInInches] drawInRect:rulerHorizontalTextRect withFont:font lineBreakMode:UILineBreakModeTailTruncation alignment:UITextAlignmentCenter];
    
    CGPoint originShift = CGPointMake(rulerVerticalTextRect.origin.x, rulerVerticalTextRect.origin.y - rulerVerticalTextRect.size.height * 0.2);
    
    CGContextTranslateCTM(context, originShift.x, originShift.y);
    CGContextConcatCTM(context, CGAffineTransformMakeRotation(1.57));
    CGContextTranslateCTM(context, -originShift.x, -originShift.y);
    
    [[NSString stringWithFormat:@"%f in", heightInInches] drawInRect:rulerVerticalTextRect withFont:font lineBreakMode:UILineBreakModeTailTruncation alignment:UITextAlignmentCenter];
     
    CGContextRestoreGState(context);
}

@end
