//
//  Barcode_MobileBarcodeViewController.m
//  Barcode Mobile
//
//  Created by Aaron Wright on 5/2/12.
//  Copyright (c) 2012 A.C. Wright Design. All rights reserved.
//

#import "Barcode_MobileBarcodeViewController.h"
#import "Barcode_MobileAppDelegate.h"

@implementation Barcode_MobileBarcodeViewController

@synthesize managedObjectContext = _managedObjectContext;
@synthesize barcodeView = _barcodeView;
@synthesize toolbar = _toolbar;
@synthesize toggleButton = _toggleButton;
@synthesize infoButton = _infoButton;
@synthesize actionButton = _actionButton;
@synthesize barcode = _barcode;
@synthesize actionSheet = _actionSheet;
@synthesize printInteractionController = _printInteractionController;
@synthesize infoPopover = _infoPopover;

# pragma mark -
# pragma mark view

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(receiveBarcodeTableViewDidChangeSelectionNotification:) 
                                                 name:kBarcode_MobileBarcodeTableViewControllerSelectionChangedNotification 
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(update:) 
                                                 name:NSManagedObjectContextObjectsDidChangeNotification 
                                               object:_managedObjectContext];
    [[NSNotificationCenter defaultCenter] addObserver:self 
                                             selector:@selector(update:) 
                                                 name:NSUserDefaultsDidChangeNotification 
                                               object:nil];
}

- (void)viewDidUnload {
    [super viewDidUnload];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self 
                                                    name:kBarcode_MobileBarcodeTableViewControllerSelectionChangedNotification 
                                                  object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self 
                                                    name:NSManagedObjectContextObjectsDidChangeNotification 
                                                  object:_managedObjectContext];
    [[NSNotificationCenter defaultCenter] removeObserver:self 
                                                    name:NSUserDefaultsDidChangeNotification 
                                                  object:nil];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return YES;
}

# pragma mark -
# pragma mark actions

- (void)update:(id)sender {
    BarcodeKitBarcode *barcode = [[BarcodeKitBarcodeManager defaultManager] barcodeForSymbology:[_barcode symbology]];
    [barcode setAlpha:[_barcode alpha]];
    [barcode setBarWidth:[_barcode barWidth]];
    [barcode setBarHeight:[_barcode barHeight]];
    [barcode setCaptionHeight:[_barcode captionHeight]];
    [barcode setCaptionSize:[_barcode captionSize]];
    [barcode setCaptioned:[_barcode captioned]];
    [barcode setColor:[UIColor colorFromHexadecimalValue:[_barcode color]]];
    [barcode setData:[_barcode data]];
    
    [_barcodeView setShouldShowRulers:[[NSUserDefaults standardUserDefaults] boolForKey:@"showRulers"]];
    [_barcodeView setBarcode:barcode];
}

- (void)toggle:(id)sender {
    [(Barcode_MobileAppDelegate *)[[UIApplication sharedApplication] delegate] toggle:sender];
    [self update:sender];
}

- (void)action:(id)sender {
    if (_printInteractionController) {
        [_printInteractionController dismissAnimated:YES];
        _printInteractionController = nil;
    }
    
    if (_actionSheet) {
        [_actionSheet dismissWithClickedButtonIndex:_actionSheet.cancelButtonIndex animated:YES];
        _actionSheet = nil;
        return;
    } else {
        _actionSheet = [[UIActionSheet alloc] init];
        [_actionSheet addButtonWithTitle:@"E-Mail"];
        [_actionSheet addButtonWithTitle:@"Twitter"];
        [_actionSheet addButtonWithTitle:@"Print"];
        [_actionSheet setDelegate:self];
        [_actionSheet showFromBarButtonItem:sender animated:YES];
    }
}

- (void)email:(id)sender {
    if ([MFMailComposeViewController canSendMail]) {
        
        BarcodeKitBarcode *barcode = [[BarcodeKitBarcodeManager defaultManager] barcodeForSymbology:[_barcode symbology]];
        [barcode setAlpha:[_barcode alpha]];
        [barcode setBarWidth:[_barcode barWidth]];
        [barcode setBarHeight:[_barcode barHeight]];
        [barcode setCaptionHeight:[_barcode captionHeight]];
        [barcode setCaptionSize:[_barcode captionSize]];
        [barcode setCaptioned:[_barcode captioned]];
        [barcode setColor:[UIColor colorFromHexadecimalValue:[_barcode color]]];
        [barcode setData:[_barcode data]];
        
        BarcodeKitBarcodeView *barcodeView = [[BarcodeKitBarcodeView alloc] initWithFrame:CGRectZero];
        [barcodeView setBarcode:barcode];
        
        MFMailComposeViewController *mailViewController = [[MFMailComposeViewController alloc] init];
        [mailViewController setMailComposeDelegate:self];
        [mailViewController setSubject:@"Check out this sweet barcode!"];
        [mailViewController addAttachmentData:[barcodeView PNGData] mimeType:@"image/png" fileName:@"Barcode"];
        
        [self presentModalViewController:mailViewController animated:YES];
    }
}

- (void)tweet:(id)sender {
    if ([TWTweetComposeViewController canSendTweet]) {
        BarcodeKitBarcode *barcode = [[BarcodeKitBarcodeManager defaultManager] barcodeForSymbology:[_barcode symbology]];
        [barcode setAlpha:[_barcode alpha]];
        [barcode setBarWidth:[_barcode barWidth]];
        [barcode setBarHeight:[_barcode barHeight]];
        [barcode setCaptionHeight:[_barcode captionHeight]];
        [barcode setCaptionSize:[_barcode captionSize]];
        [barcode setCaptioned:[_barcode captioned]];
        [barcode setColor:[UIColor colorFromHexadecimalValue:[_barcode color]]];
        [barcode setData:[_barcode data]];
        
        BarcodeKitBarcodeView *barcodeView = [[BarcodeKitBarcodeView alloc] initWithFrame:CGRectZero];
        [barcodeView setBarcode:barcode];
        
        TWTweetComposeViewController *tweetSheet = [[TWTweetComposeViewController alloc] init];
        [tweetSheet addImage:[UIImage imageWithData:[barcodeView PNGData]]];
        [tweetSheet setInitialText:@"Check out this sweet barcode!"];
        [self presentModalViewController:tweetSheet animated:YES];
    } else {
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Sorry"
                                                            message:@"You can't send a tweet right now, make sure your device has an internet connection and you have at least one Twitter account setup."                                                          
                                                           delegate:self                                             
                                                  cancelButtonTitle:@"OK"                                           
                                                  otherButtonTitles:nil];
        [alertView show];
    }
}

- (void)print:(id)sender {
    if (_printInteractionController) {
        [_printInteractionController dismissAnimated:YES];
        _printInteractionController = nil;
        return;
    } else {
        BarcodeKitBarcode *barcode = [[BarcodeKitBarcodeManager defaultManager] barcodeForSymbology:[_barcode symbology]];
        [barcode setAlpha:[_barcode alpha]];
        [barcode setBarWidth:[_barcode barWidth]];
        [barcode setBarHeight:[_barcode barHeight]];
        [barcode setCaptionHeight:[_barcode captionHeight]];
        [barcode setCaptionSize:[_barcode captionSize]];
        [barcode setCaptioned:[_barcode captioned]];
        [barcode setColor:[UIColor colorFromHexadecimalValue:[_barcode color]]];
        [barcode setData:[_barcode data]];
        
        BarcodeKitBarcodeView *barcodeView = [[BarcodeKitBarcodeView alloc] initWithFrame:CGRectZero];
        [barcodeView setBarcode:barcode];
        
        _printInteractionController = [UIPrintInteractionController sharedPrintController];
        _printInteractionController.delegate = self;
        
        UIPrintInfo *printInfo = [UIPrintInfo printInfo];
        printInfo.outputType = UIPrintInfoOutputGeneral;
        printInfo.jobName = [_barcode title];
        
        _printInteractionController.printInfo = printInfo;
        _printInteractionController.printingItem = [barcodeView PDFData];
        
        [_printInteractionController presentFromBarButtonItem:_actionButton animated:YES completionHandler:^(UIPrintInteractionController *printController, BOOL completed, NSError *error) {
            if (!completed && error) {
                NSLog(@"Printing could not complete because of error: %@", error);
            }
        }];
    }
}

- (IBAction)showInfoPopover:(id)sender {
    if (_infoPopover) {
        [_infoPopover dismissPopoverAnimated:YES];
    } else {
        [self performSegueWithIdentifier:@"Info" sender:sender];
    }
}

- (void)setBarcode:(Barcode_MobileBarcode *)barcode {
    _barcode = barcode;
    
    if (_barcode) {
        [_infoButton setEnabled:YES];
    } else {
        [_infoButton setEnabled:NO];
    }
    
    [self update:self];
}

# pragma mark -
# pragma mark notifications

- (void)receiveBarcodeTableViewDidChangeSelectionNotification:(NSNotification *)notification {
    [self setBarcode:[notification object]];
}

# pragma mark -
# pragma mark segue

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([[segue identifier] isEqualToString:@"Info"]) {
        [(Barcode_MobileInfoTableViewController *)[[segue destinationViewController] topViewController] setBarcode:[self barcode]];
        [(Barcode_MobileInfoTableViewController *)[[segue destinationViewController] topViewController] setManagedObjectContext:[self managedObjectContext]];
        _infoPopover = [(UIStoryboardPopoverSegue *)segue popoverController];
    }
}

# pragma mark -
# pragma mark UISplitViewDelegate

- (void)splitViewController:(UISplitViewController *)svc willHideViewController:(UIViewController *)aViewController withBarButtonItem:(UIBarButtonItem *)barButtonItem forPopoverController:(UIPopoverController *)pc {
    NSMutableArray *items = [[self.toolbar items] mutableCopy];
    
    if ( barButtonItem == nil ) return;
    
    barButtonItem.title = @"Barcodes";
    
    [items removeObjectAtIndex:0];
    [items insertObject:barButtonItem atIndex:0];
    [self.toolbar setItems:items animated:NO];
    
    [self update:self];
}

- (void)splitViewController:(UISplitViewController *)svc willShowViewController:(UIViewController *)aViewController invalidatingBarButtonItem:(UIBarButtonItem *)button {
    NSMutableArray *items = [[self.toolbar items] mutableCopy];
    
    [items removeObjectAtIndex:0];
    [items insertObject:_toggleButton atIndex:0];
    [self.toolbar setItems:items animated:NO];
    
    [self update:self];
}

# pragma mark -
# pragma mark UIActionSheetDelegate

- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
    switch (buttonIndex) {
        case 0:
            [self email:_actionButton];
            break;
        case 1:
            [self tweet:_actionButton];
            break;
        case 2:
            [self print:_actionButton];
            break;
        default:
            break;
    }
}

# pragma mark -
# pragma mark MFMailComposeViewControllerDelegate

- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error {
    [self dismissModalViewControllerAnimated:YES];
}

@end
