//
//  Barcode_MobileInfoTableViewController.m
//  Barcode Mobile
//
//  Created by Aaron Wright on 5/16/12.
//  Copyright (c) 2012 A.C. Wright Design. All rights reserved.
//

#import "Barcode_MobileInfoTableViewController.h"
#import "Barcode_MobileAppDelegate.h"

@implementation Barcode_MobileInfoTableViewController

@synthesize managedObjectContext = _managedObjectContext;
@synthesize barcode = _barcode;
@synthesize titleTextField = _titleTextField;
@synthesize dataTextField = _dataTextField;
@synthesize colorTextField = _colorTextField;
@synthesize widthSlider = _widthSlider;
@synthesize heightSlider = _heightSlider;
@synthesize alphaSlider = _alphaSlider;
@synthesize widthLabel = _widthLabel;
@synthesize heightLabel = _heightLabel;
@synthesize captionHeightLabel = _captionHeightLabel;
@synthesize captionSizeLabel = _captionSizeLabel;
@synthesize alphaLabel = _alphaLabel;
@synthesize symbologyLabel = _symbologyLabel;
@synthesize captionSwitch = _captionSwitch;
@synthesize integerFormatter = _integerFormatter;
@synthesize floatFormatter = _floatFormatter;

# pragma mark -
# pragma mark awake

- (void)awakeFromNib {
    NSNumberFormatter *integerFormatter = [[NSNumberFormatter alloc] init];
    [integerFormatter setFormatterBehavior:NSNumberFormatterBehaviorDefault]; 
    [integerFormatter setNumberStyle:NSNumberFormatterNoStyle];
    [integerFormatter setRoundingMode:NSNumberFormatterRoundHalfUp];
    
    NSNumberFormatter *floatFormatter = [[NSNumberFormatter alloc] init];
    [floatFormatter setFormatterBehavior:NSNumberFormatterBehaviorDefault]; 
    [floatFormatter setNumberStyle:NSNumberFormatterDecimalStyle];
    [floatFormatter setMaximumSignificantDigits:1];
    [floatFormatter setMaximumFractionDigits:1];
    [floatFormatter setMinimumFractionDigits:1];
    
    [self setIntegerFormatter:integerFormatter];
    [self setFloatFormatter:floatFormatter];
}

# pragma mark -
# pragma mark view

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    [_titleTextField setText:[_barcode title]];
    [_dataTextField setText:[_barcode data]];
    [_colorTextField setText:[_barcode color]];
    [_widthSlider setValue:[[_barcode barWidth] floatValue]];
    [_heightSlider setValue:[[_barcode barHeight] floatValue]];
    [_captionHeightSlider setValue:[[_barcode captionHeight] floatValue]];
    [_captionSizeSlider setValue:[[_barcode captionSize] floatValue]];
    [_alphaSlider setValue:[[_barcode alpha] floatValue]];
    [_widthLabel setText:[_integerFormatter stringFromNumber:[_barcode barWidth]]];
    [_heightLabel setText:[_integerFormatter stringFromNumber:[_barcode barHeight]]];
    [_captionHeightLabel setText:[_integerFormatter stringFromNumber:[_barcode captionHeight]]];
    [_captionSizeLabel setText:[_integerFormatter stringFromNumber:[_barcode captionSize]]];
    [_alphaLabel setText:[_floatFormatter stringFromNumber:[_barcode alpha]]];
    [_symbologyLabel setText:[_barcode symbology]];
    [_captionSwitch setOn:[_barcode captioned]];
}

- (void)viewDidUnload {
    [super viewDidUnload];
}

- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    [(Barcode_MobileAppDelegate *)[[UIApplication sharedApplication] delegate] save:self];
}

# pragma mark -
# pragma mark actions

- (IBAction)widthSliderChanged:(id)sender {
    NSNumber *value = [NSNumber numberWithFloat:[(UISlider *)sender value]];
    [_barcode setBarWidth:value];
    [_widthLabel setText:[_integerFormatter stringFromNumber:value]];
}

- (IBAction)heightSliderChanged:(id)sender {
    NSNumber *value = [NSNumber numberWithFloat:[(UISlider *)sender value]];
    [_barcode setBarHeight:value];
    [_heightLabel setText:[_integerFormatter stringFromNumber:value]];
}

- (IBAction)captionHeightSliderChanged:(id)sender {
    NSNumber *value = [NSNumber numberWithFloat:[(UISlider *)sender value]];
    [_barcode setCaptionHeight:value];
    [_captionHeightLabel setText:[_integerFormatter stringFromNumber:value]];
}

- (IBAction)captionSizeSliderChanged:(id)sender {
    NSNumber *value = [NSNumber numberWithFloat:[(UISlider *)sender value]];
    [_barcode setCaptionSize:value];
    [_captionSizeLabel setText:[_integerFormatter stringFromNumber:value]];
}

- (IBAction)alphaSliderChanged:(id)sender {
    NSNumber *value = [NSNumber numberWithFloat:[(UISlider *)sender value]];
    [_barcode setAlpha:value];
    [_alphaLabel setText:[_floatFormatter stringFromNumber:value]];
}

- (IBAction)captionSwitchChanged:(id)sender {
    [_barcode setCaptioned:[(UISwitch *)sender isOn]];
}

# pragma mark -
# pragma mark UITextFieldDelegate

- (void)textFieldDidEndEditing:(UITextField *)textField {
    if ([textField isEqual:_titleTextField]) {
        [_barcode setTitle:[_titleTextField text]];
    }
    if ([textField isEqual:_dataTextField]) {
        [_barcode setData:[_dataTextField text]];
    }
    if ([textField isEqual:_colorTextField]) {
        [_barcode setColor:[_colorTextField text]];
    }
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    if ([textField isEqual:_titleTextField] ||
        [textField isEqual:_colorTextField] ||
        [textField isEqual:_colorTextField]) {
        [textField resignFirstResponder];
    }
    return NO;
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if ([[segue identifier] isEqualToString:@"Symbology"]) {
        [(Barcode_MobileSymbologyViewController *)[segue destinationViewController] setBarcode:_barcode];
    }
}

@end
